@@ -71,7 +71,7 @@
 
 // See online_source_is_failing()
 #define ONLINE_SEARCH_COOLDOWN_TIME 3600
-#define ONLINE_SEARCH_FAILURES_MAX 3
+#define ONLINE_SEARCH_FAILURES_MAX 5
 
 enum artwork_cache
 {
@@ -192,6 +192,8 @@ static const char *cover_extension[] =
     "jpg", "png",
   };
 
+static pthread_mutex_t artwork_cache_stash_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 /* ----------------- DECLARE AND CONFIGURE SOURCE HANDLERS ----------------- */
 
 /* Forward - group handlers */
@@ -336,6 +338,13 @@ static struct artwork_source artwork_item_source[] =
       .media_kinds = MEDIA_KIND_MUSIC,
       .cache = NEVER,
     },
+    {
+      .name = "own (pipe)",
+      .handler = source_item_own_get,
+      .data_kinds = (1 << DATA_KIND_PIPE),
+      .media_kinds = MEDIA_KIND_ALL,
+      .cache = ON_SUCCESS | ON_FAILURE,
+    },
     {
       .name = NULL,
       .handler = NULL,
@@ -450,7 +459,6 @@ artwork_read_byurl(struct evbuffer *evbuf, const char *url)
   ret = http_client_request(&client, NULL);
   if (ret < 0)
     {
-      DPRINTF(E_LOG, L_ART, "Request to '%s' failed with return value %d\n", url, ret);
       goto out;
     }
 
@@ -582,20 +590,6 @@ size_calculate(int *dst_w, int *dst_h, int src_w, int src_h, int max_w, int max_
   DPRINTF(E_DBG, L_ART, "Rescale required, destination width %d height %d\n", *dst_w, *dst_h);
 }
 
-#ifdef HAVE_LIBEVENT2_OLD
-// This is not how this function is actually defined in libevent 2.1+, but it
-// works as a less optimal stand-in
-int
-evbuffer_add_buffer_reference(struct evbuffer *outbuf, struct evbuffer *inbuf)
-{
-  uint8_t *buf = evbuffer_pullup(inbuf, -1);
-  if (!buf)
-    return -1;
-
-  return evbuffer_add_reference(outbuf, buf, evbuffer_get_length(inbuf), NULL, NULL);
-}
-#endif
-
 /*
  * Either gets the artwork file given in "path" (rescaled if needed) or rescales
  * the artwork given in "inbuf".
@@ -611,6 +605,8 @@ evbuffer_add_buffer_reference(struct evbuffer *outbuf, struct evbuffer *inbuf)
 static int
 artwork_get(struct evbuffer *evbuf, char *path, struct evbuffer *in_buf, bool is_embedded, enum data_kind data_kind, struct artwork_req_params req_params)
 {
+  struct transcode_decode_setup_args xcode_decode_args = { .profile = XCODE_JPEG }; // Covers XCODE_PNG too
+  struct transcode_encode_setup_args xcode_encode_args = { 0 };
   struct decode_ctx *xcode_decode = NULL;
   struct encode_ctx *xcode_encode = NULL;
   struct transcode_evbuf_io xcode_evbuf_io = { 0 };
@@ -644,13 +640,16 @@ artwork_get(struct evbuffer *evbuf, char *path, struct evbuffer *in_buf, bool is
 	}
 
       xcode_evbuf_io.evbuf = xcode_buf;
-      xcode_decode = transcode_decode_setup(XCODE_JPEG, NULL, data_kind, NULL, &xcode_evbuf_io, 0); // Covers XCODE_PNG too
+      xcode_decode_args.evbuf_io = &xcode_evbuf_io;
+      xcode_decode_args.is_http = (data_kind == DATA_KIND_HTTP);
     }
   else
     {
-      xcode_decode = transcode_decode_setup(XCODE_JPEG, NULL, data_kind, path, NULL, 0); // Covers XCODE_PNG too
+      xcode_decode_args.path = path;
+      xcode_decode_args.is_http = (data_kind == DATA_KIND_HTTP);
     }
 
+  xcode_decode = transcode_decode_setup(xcode_decode_args);
   if (!xcode_decode)
     {
       if (path)
@@ -709,15 +708,19 @@ artwork_get(struct evbuffer *evbuf, char *path, struct evbuffer *in_buf, bool is
       goto out;
     }
 
+  xcode_encode_args.src_ctx = xcode_decode;
+  xcode_encode_args.width = dst_width;
+  xcode_encode_args.height = dst_height;
   if (dst_format == ART_FMT_JPEG)
-    xcode_encode = transcode_encode_setup(XCODE_JPEG, NULL, xcode_decode, NULL, dst_width, dst_height);
+    xcode_encode_args.profile = XCODE_JPEG;
   else if (dst_format == ART_FMT_PNG)
-    xcode_encode = transcode_encode_setup(XCODE_PNG, NULL, xcode_decode, NULL, dst_width, dst_height);
+    xcode_encode_args.profile = XCODE_PNG;
   else if (dst_format == ART_FMT_VP8)
-    xcode_encode = transcode_encode_setup(XCODE_VP8, NULL, xcode_decode, NULL, dst_width, dst_height);
+    xcode_encode_args.profile = XCODE_VP8;
   else
-    xcode_encode = transcode_encode_setup(XCODE_JPEG, NULL, xcode_decode, NULL, dst_width, dst_height);
+    xcode_encode_args.profile = XCODE_JPEG;
 
+  xcode_encode = transcode_encode_setup(xcode_encode_args);
   if (!xcode_encode)
     {
       if (path)
@@ -925,7 +928,7 @@ artwork_get_bydir(struct evbuffer *evbuf, char *out_path, size_t len, char *dir,
  * before making a request. Stashes result in cache, also if negative.
  *
  * @out artwork   Image data
- * @in  url       URL of the artwork
+ * @in  url       HTTP(S) URL of the artwork
  * @in  req_params Requested max size/format
  * @return        ART_FMT_* on success, ART_E_NONE or ART_E_ERROR
  */
@@ -939,12 +942,18 @@ artwork_get_byurl(struct evbuffer *artwork, const char *url, struct artwork_req_
   CHECK_NULL(L_ART, raw = evbuffer_new());
   format = ART_E_ERROR;
 
+  // Accessing the cache is thread safe, the purpose of the lock is to make the
+  // artwork stash more effective if we have parallel requests for the same url.
+  // It will assure that the artwork from the first request is downloaded and
+  // stashed before processing the next request.
+  pthread_mutex_lock(&artwork_cache_stash_mutex);
   ret = cache_artwork_read(raw, url, &format);
   if (ret < 0)
     {
       format = artwork_read_byurl(raw, url);
       cache_artwork_stash(raw, url, format);
     }
+  pthread_mutex_unlock(&artwork_cache_stash_mutex);
 
   // If we couldn't read, or we have cached a negative result from the last attempt, we stop now
   if (format <= 0)
@@ -1121,7 +1130,7 @@ online_source_response_parse(char **artwork_url, const struct online_source *src
 }
 
 static int
-online_source_request_url_make(char *url, size_t url_size, const struct online_source *src, struct artwork_ctx *ctx)
+online_source_search_url_make(char *url, size_t url_size, const struct online_source *src, struct artwork_ctx *ctx)
 {
   struct db_queue_item *queue_item;
   struct keyval query = { 0 };
@@ -1164,6 +1173,13 @@ online_source_request_url_make(char *url, size_t url_size, const struct online_s
       goto error;
     }
 
+  if ((artist && strncmp(CFG_NAME_UNKNOWN_ARTIST, artist, strlen(CFG_NAME_UNKNOWN_ARTIST)) == 0) ||
+      (album && strncmp(CFG_NAME_UNKNOWN_ALBUM, album, strlen(CFG_NAME_UNKNOWN_ARTIST)) == 0) )
+    {
+      DPRINTF(E_DBG, L_ART, "Skipping online artwork search for unknown artist/album\n");
+      goto error;
+    }
+
   for (i = 0; src->query_parts[i].key; i++)
     {
       if (!album && strstr(src->query_parts[i].template, "$ALBUM$"))
@@ -1183,14 +1199,17 @@ online_source_request_url_make(char *url, size_t url_size, const struct online_s
       ret = keyval_add(&query, src->query_parts[i].key, param);
       if (ret < 0)
 	{
-	  DPRINTF(E_LOG, L_ART, "keyval_add() failed in request_url_make()\n");
+	  DPRINTF(E_LOG, L_ART, "keyval_add() failed in online_source_request_url_make()\n");
 	  goto error;
 	}
     }
 
   encoded_query = http_form_urlencode(&query);
   if (!encoded_query)
-    goto error;
+    {
+      DPRINTF(E_WARN, L_ART, "URL encoding for online artwork search failed\n");
+      goto error;
+    }
 
   snprintf(url, url_size, "%s?%s", src->search_endpoint, src->search_param);
   if (safe_snreplace(url, url_size, "$QUERY$", encoded_query) < 0)
@@ -1218,18 +1237,13 @@ online_source_search_check_last(char **last_artwork_url, const struct online_sou
   struct online_search_history *history = src->search_history;
   bool is_same;
 
-  pthread_mutex_lock(&history->mutex);
-
   is_same = (hash == history->last_hash) &&
             (max_w == history->last_max_w) &&
             (max_h == history->last_max_h);
 
-  // Copy this to the caller while we have the lock anyway
   if (is_same)
     *last_artwork_url = safe_strdup(history->last_artwork_url);
 
-  pthread_mutex_unlock(&history->mutex);
-
   return is_same ? 0 : -1;
 }
 
@@ -1239,8 +1253,6 @@ online_source_is_failing(const struct online_source *src, int id)
   struct online_search_history *history = src->search_history;
   bool is_failing;
 
-  pthread_mutex_lock(&history->mutex);
-
   // If the last request was more than ONLINE_SEARCH_COOLDOWN_TIME ago we will always try again
   if (time(NULL) > history->last_timestamp + ONLINE_SEARCH_COOLDOWN_TIME)
     is_failing = false;
@@ -1257,22 +1269,20 @@ online_source_is_failing(const struct online_source *src, int id)
   else
     is_failing = true;
 
-  pthread_mutex_unlock(&history->mutex);
-
   return is_failing;
 }
 
 static void
-online_source_history_update(const struct online_source *src, int id, uint32_t request_hash, int response_code, const char *artwork_url)
+online_source_history_update(const struct online_source *src, int id, uint32_t request_hash, int response_code, const char *artwork_url, int max_w, int max_h)
 {
   struct online_search_history *history = src->search_history;
 
-  pthread_mutex_lock(&history->mutex);
-
   history->last_id = id;
   history->last_hash = request_hash;
   history->last_response_code = response_code;
   history->last_timestamp = time(NULL);
+  history->last_max_w = max_w;
+  history->last_max_h = max_h;
 
   free(history->last_artwork_url);
   history->last_artwork_url = safe_strdup(artwork_url); // FIXME should free this on exit
@@ -1281,14 +1291,12 @@ online_source_history_update(const struct online_source *src, int id, uint32_t r
     history->count_failures = 0;
   else
     history->count_failures++;
-
-  pthread_mutex_unlock(&history->mutex);
 }
 
 static int
 auth_header_add(struct keyval *headers, const struct online_source *src)
 {
-  char auth_header[256];
+  char auth_header[512];
   char *auth_key;
   char *auth_secret;
   int ret;
@@ -1318,34 +1326,24 @@ auth_header_add(struct keyval *headers, const struct online_source *src)
 }
 
 static char *
-online_source_search(const struct online_source *src, struct artwork_ctx *ctx)
+online_source_artwork_url_get(const char *search_url, const struct online_source *src, int id, int max_w, int max_h)
 {
   char *artwork_url;
   struct http_client_ctx client = { 0 };
   struct keyval output_headers = { 0 };
   uint32_t hash;
-  char url[2048];
   int ret;
 
-  DPRINTF(E_SPAM, L_ART, "Trying %s for %s\n", src->name, ctx->dbmfi->path);
-
-  ret = online_source_request_url_make(url, sizeof(url), src, ctx);
-  if (ret < 0)
-    {
-      DPRINTF(E_WARN, L_ART, "Skipping artwork source %s, could not construct a request URL\n", src->name);
-      return NULL;
-    }
-
   // Be nice to our peer + improve response times by not repeating search requests
-  hash = djb_hash(url, strlen(url));
-  ret = online_source_search_check_last(&artwork_url, src, hash, ctx->req_params.max_w, ctx->req_params.max_h);
+  hash = djb_hash(search_url, strlen(search_url));
+  ret = online_source_search_check_last(&artwork_url, src, hash, max_w, max_h);
   if (ret == 0)
     {
       return artwork_url; // Will be NULL if we are repeating a search that failed
     }
 
   // If our recent searches have been futile we may give the source a break
-  if (online_source_is_failing(src, ctx->id))
+  if (online_source_is_failing(src, id))
     {
       DPRINTF(E_DBG, L_ART, "Skipping artwork source %s, too many failed requests\n", src->name);
       return NULL;
@@ -1358,18 +1356,18 @@ online_source_search(const struct online_source *src, struct artwork_ctx *ctx)
     }
 
   CHECK_NULL(L_ART, client.input_body = evbuffer_new());
-  client.url = url;
+  client.url = search_url;
   client.output_headers = &output_headers;
 
   ret = http_client_request(&client, NULL);
   keyval_clear(&output_headers);
   if (ret < 0 || client.response_code != HTTP_OK)
     {
-      DPRINTF(E_WARN, L_ART, "Artwork request to '%s' failed, response code %d\n", url, client.response_code);
+      DPRINTF(E_WARN, L_ART, "Artwork request to '%s' failed, response code %d\n", search_url, client.response_code);
       goto error;
     }
 
-  ret = online_source_response_parse(&artwork_url, src, client.input_body, ctx->req_params.max_w, ctx->req_params.max_h);
+  ret = online_source_response_parse(&artwork_url, src, client.input_body, max_w, max_h);
   if (ret == ONLINE_SOURCE_PARSE_NOT_FOUND)
     DPRINTF(E_DBG, L_ART, "No image tag found in response from source '%s'\n", src->name);
   else if (ret == ONLINE_SOURCE_PARSE_INVALID)
@@ -1382,16 +1380,41 @@ online_source_search(const struct online_source *src, struct artwork_ctx *ctx)
   if (ret != ONLINE_SOURCE_PARSE_OK)
     goto error;
 
-  online_source_history_update(src, ctx->id, hash, client.response_code, artwork_url);
+  online_source_history_update(src, id, hash, client.response_code, artwork_url, max_w, max_h);
   evbuffer_free(client.input_body);
   return artwork_url;
 
  error:
-  online_source_history_update(src, ctx->id, hash, client.response_code, NULL);
+  online_source_history_update(src, id, hash, client.response_code, NULL, max_w, max_h);
   evbuffer_free(client.input_body);
   return NULL;
 }
 
+static char *
+online_source_search(const struct online_source *src, struct artwork_ctx *ctx)
+{
+  struct online_search_history *history = src->search_history;
+  char search_url[2048];
+  char *artwork_url;
+  int ret;
+
+  DPRINTF(E_SPAM, L_ART, "Trying %s for %s\n", src->name, ctx->dbmfi->path);
+
+  ret = online_source_search_url_make(search_url, sizeof(search_url), src, ctx);
+  if (ret < 0)
+    return NULL;
+
+  // The protection against flooding the online source with requests requires
+  // that online_source_request() isn't called in parallel
+  pthread_mutex_lock(&history->mutex);
+
+  artwork_url = online_source_artwork_url_get(search_url, src, ctx->id, ctx->req_params.max_w, ctx->req_params.max_h);
+
+  pthread_mutex_unlock(&history->mutex);
+
+  return artwork_url;
+}
+
 static bool
 online_source_is_enabled(const struct online_source *src)
 {
@@ -1591,16 +1614,22 @@ static int
 source_item_artwork_url_get(struct artwork_ctx *ctx)
 {
   struct db_queue_item *queue_item;
+  const char *proto_http = "http:";
+  const char *proto_https = "https:";
+  bool is_http;
+  bool is_https;
   int ret;
 
   DPRINTF(E_SPAM, L_ART, "Trying artwork url for %s\n", ctx->dbmfi->path);
 
   queue_item = db_queue_fetch_byfileid(ctx->id);
   if (!queue_item || !queue_item->artwork_url)
-    {
-      free_queue_item(queue_item, 0);
-      return ART_E_NONE;
-    }
+    goto notfound;
+
+  is_http = (strncmp(queue_item->artwork_url, proto_http, strlen(proto_http)) == 0);
+  is_https = (strncmp(queue_item->artwork_url, proto_https, strlen(proto_https)) == 0);
+  if (!is_http && !is_https)
+    goto notfound;
 
   ret = artwork_get_byurl(ctx->evbuf, queue_item->artwork_url, ctx->req_params);
 
@@ -1609,6 +1638,10 @@ source_item_artwork_url_get(struct artwork_ctx *ctx)
   free_queue_item(queue_item, 0);
 
   return ret;
+
+ notfound:
+  free_queue_item(queue_item, 0);
+  return ART_E_NONE;
 }
 
 /*
@@ -1620,17 +1653,22 @@ static int
 source_item_pipe_get(struct artwork_ctx *ctx)
 {
   struct db_queue_item *queue_item;
-  const char *proto = "file:";
+  const char *proto_file = "file:";
+  bool is_file;
   char *path;
   int ret;
 
   DPRINTF(E_SPAM, L_ART, "Trying pipe metadata from %s.metadata\n", ctx->dbmfi->path);
 
   queue_item = db_queue_fetch_byfileid(ctx->id);
-  if (!queue_item || !queue_item->artwork_url || strncmp(queue_item->artwork_url, proto, strlen(proto)) != 0)
+  if (!queue_item || !queue_item->artwork_url)
     goto notfound;
 
-  path = queue_item->artwork_url + strlen(proto);
+  is_file = (strncmp(queue_item->artwork_url, proto_file, strlen(proto_file)) == 0);
+  if (!is_file)
+    goto notfound;
+
+  path = queue_item->artwork_url + strlen(proto_file);
 
   // Sometimes the file has been replaced, but queue_item->artwork_url hasn't
   // been updated yet. In that case just stop now.
