@@ -49,13 +49,15 @@ static cfg_opt_t sec_general[] =
     CFG_STR("db_backup_path", NULL, CFGF_NONE),
     CFG_STR("logfile", STATEDIR "/log/" PACKAGE ".log", CFGF_NONE),
     CFG_INT_CB("loglevel", E_LOG, CFGF_NONE, &cb_loglevel),
+    CFG_STR("logformat", "default", CFGF_NONE),
     CFG_STR("admin_password", NULL, CFGF_NONE),
     CFG_INT("websocket_port", 3688, CFGF_NONE),
     CFG_STR("websocket_interface", NULL, CFGF_NONE),
-    CFG_STR_LIST("trusted_networks", "{localhost,192.168,fd}", CFGF_NONE),
-    CFG_BOOL("ipv6", cfg_true, CFGF_NONE),
+    CFG_STR_LIST("trusted_networks", "{lan}", CFGF_NONE),
+    CFG_BOOL("ipv6", cfg_false, CFGF_NONE),
     CFG_STR("bind_address", NULL, CFGF_NONE),
-    CFG_STR("cache_path", STATEDIR "/cache/" PACKAGE "/cache.db", CFGF_NONE),
+    CFG_STR("cache_dir", STATEDIR "/cache/" PACKAGE, CFGF_NONE),
+    CFG_STR("cache_path", NULL, CFGF_DEPRECATED),
     CFG_INT("cache_daap_threshold", 1000, CFGF_NONE),
     CFG_BOOL("speaker_autoselect", cfg_false, CFGF_NONE),
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
@@ -67,6 +69,9 @@ static cfg_opt_t sec_general[] =
     CFG_INT("db_pragma_cache_size", -1, CFGF_NONE),
     CFG_STR("db_pragma_journal_mode", NULL, CFGF_NONE),
     CFG_INT("db_pragma_synchronous", -1, CFGF_NONE),
+    CFG_STR("cache_daap_filename", "daap.db", CFGF_NONE),
+    CFG_STR("cache_artwork_filename", "artwork.db", CFGF_NONE),
+    CFG_STR("cache_xcode_filename", "xcode.db", CFGF_NONE),
     CFG_STR("allow_origin", "*", CFGF_NONE),
     CFG_STR("user_agent", PACKAGE_NAME "/" PACKAGE_VERSION, CFGF_NONE),
     CFG_BOOL("ssl_verifypeer", cfg_true, CFGF_NONE),
@@ -111,10 +116,14 @@ static cfg_opt_t sec_library[] =
     CFG_BOOL("itunes_smartpl", cfg_false, CFGF_NONE),
     CFG_STR_LIST("no_decode", NULL, CFGF_NONE),
     CFG_STR_LIST("force_decode", NULL, CFGF_NONE),
+    CFG_STR("prefer_format", NULL, CFGF_NONE),
     CFG_BOOL("pipe_autostart", cfg_true, CFGF_NONE),
     CFG_INT("pipe_sample_rate", 44100, CFGF_NONE),
     CFG_INT("pipe_bits_per_sample", 16, CFGF_NONE),
     CFG_BOOL("rating_updates", cfg_false, CFGF_NONE),
+    CFG_BOOL("read_rating", cfg_false, CFGF_NONE),
+    CFG_BOOL("write_rating", cfg_false, CFGF_NONE),
+    CFG_INT("max_rating", 100, CFGF_NONE),
     CFG_BOOL("allow_modifying_stored_playlists", cfg_false, CFGF_NONE),
     CFG_STR("default_playlist_directory", NULL, CFGF_NONE),
     CFG_BOOL("clear_queue_on_stop_disable", cfg_false, CFGF_NONE),
@@ -156,6 +165,7 @@ static cfg_opt_t sec_airplay_shared[] =
   {
     CFG_INT("control_port", 0, CFGF_NONE),
     CFG_INT("timing_port", 0, CFGF_NONE),
+    CFG_BOOL("uncompressed_alac", cfg_false, CFGF_NONE),
     CFG_END()
   };
 
@@ -208,6 +218,12 @@ static cfg_opt_t sec_spotify[] =
     CFG_BOOL("base_playlist_disable", cfg_false, CFGF_NONE),
     CFG_BOOL("artist_override", cfg_false, CFGF_NONE),
     CFG_BOOL("album_override", cfg_false, CFGF_NONE),
+    CFG_BOOL("disable_legacy_mode", cfg_false, CFGF_NONE),
+    // Issued by Spotify on developer.spotify.com for forked-daapd/OwnTone
+    CFG_STR("webapi_client_id", "0e684a5422384114a8ae7ac020f01789", CFGF_NONE),
+    CFG_STR("webapi_client_secret", "232af95f39014c9ba218285a5c11a239", CFGF_NONE),
+    // Must be in allow-list on developer.spotify.com for forked-daapd/OwnTone
+    CFG_STR("redirect_uri", "https://owntone.github.io/owntone-oauth/spotify/", CFGF_NONE),
     CFG_END()
   };
 
@@ -229,6 +245,7 @@ static cfg_opt_t sec_mpd[] =
   {
     CFG_INT("port", 6600, CFGF_NONE),
     CFG_INT("http_port", 0, CFGF_NONE),
+    CFG_BOOL("enable_httpd_plugin", cfg_false, CFGF_NONE),
     CFG_BOOL("clear_queue_on_stop_disable", cfg_false, CFGF_NODEFAULT | CFGF_DEPRECATED),
     CFG_BOOL("allow_modifying_stored_playlists", cfg_false, CFGF_NODEFAULT | CFGF_DEPRECATED),
     CFG_STR("default_playlist_directory", NULL, CFGF_NODEFAULT | CFGF_DEPRECATED),
@@ -307,6 +324,31 @@ cb_loglevel(cfg_t *config, cfg_opt_t *opt, const char *value, void *result)
   return 0;
 }
 
+// Makes sure cache_dir ends with a slash
+static int
+sanitize_cache_dir(cfg_t *general)
+{
+  char *dir;
+  const char *s;
+  char *appended;
+  size_t len;
+
+  dir = cfg_getstr(general, "cache_dir");
+  len = strlen(dir);
+
+  s = strrchr(dir, '/');
+  if (s && (s + 1 == dir + len))
+    return 0;
+
+  appended = safe_asprintf("%s/", dir);
+
+  cfg_setstr(general, "cache_dir", appended);
+
+  free(appended);
+
+  return 0;
+}
+
 static int
 conffile_expand_libname(cfg_t *lib)
 {
@@ -420,7 +462,6 @@ conffile_expand_libname(cfg_t *lib)
   return 0;
 }
 
-
 int
 conffile_load(char *file)
 {
@@ -461,6 +502,14 @@ conffile_load(char *file)
   runas_uid = pw->pw_uid;
   runas_gid = pw->pw_gid;
 
+  ret = sanitize_cache_dir(cfg_getsec(cfg, "general"));
+  if (ret != 0)
+    {
+      DPRINTF(E_FATAL, L_CONF, "Invalid configuration of cache_dir\n");
+
+      goto out_fail;
+    }
+
   lib = cfg_getsec(cfg, "library");
 
   if (cfg_size(lib, "directories") == 0)
