@@ -72,6 +72,7 @@ enum query_type {
 #define DB_ADMIN_START_TIME "start_time"
 #define DB_ADMIN_LASTFM_SESSION_KEY "lastfm_sk"
 #define DB_ADMIN_SPOTIFY_REFRESH_TOKEN "spotify_refresh_token"
+#define DB_ADMIN_LISTENBRAINZ_TOKEN "listenbrainz_token"
 
 /* Max value for media_file_info->rating (valid range is from 0 to 100) */
 #define DB_FILES_RATING_MAX 100
@@ -248,6 +249,7 @@ struct media_file_info {
   char *composer_sort;
 
   uint32_t scan_kind; /* Identifies the library_source that created/updates this item */
+  char *lyrics;
 };
 
 #define mfi_offsetof(field) offsetof(struct media_file_info, field)
@@ -422,6 +424,7 @@ struct db_media_file_info {
   char *channels;
   char *usermark;
   char *scan_kind;
+  char *lyrics;
 };
 
 #define dbmfi_offsetof(field) offsetof(struct db_media_file_info, field)
@@ -683,6 +686,9 @@ db_file_ping_bymatch(const char *path, int isdir);
 char *
 db_file_path_byid(int id);
 
+bool
+db_file_id_exists(int id);
+
 int
 db_file_id_bypath(const char *path);
 
@@ -693,7 +699,10 @@ int
 db_file_id_byurl(const char *url);
 
 int
-db_file_id_by_virtualpath_match(const char *path);
+db_file_id_byvirtualpath(const char *virtual_path);
+
+int
+db_file_id_byvirtualpath_match(const char *virtual_path);
 
 struct media_file_info *
 db_file_fetch_byid(int id);
@@ -710,15 +719,6 @@ db_file_update(struct media_file_info *mfi);
 void
 db_file_seek_update(int id, uint32_t seek);
 
-int
-db_file_rating_update_byid(uint32_t id, uint32_t rating);
-
-int
-db_file_usermark_update_byid(uint32_t id, uint32_t usermark);
-
-int
-db_file_rating_update_byvirtualpath(const char *virtual_path, uint32_t rating);
-
 void
 db_file_delete_bypath(const char *path);
 
@@ -899,12 +899,6 @@ db_queue_add_by_queryafteritemid(struct query_params *qp, uint32_t item_id);
 int
 db_queue_add_by_query(struct query_params *qp, char reshuffle, uint32_t item_id, int position, int *count, int *new_item_id);
 
-int
-db_queue_add_by_playlistid(int plid, char reshuffle, uint32_t item_id, int position, int *count, int *new_item_id);
-
-int
-db_queue_add_by_fileid(int id, char reshuffle, uint32_t item_id, int position, int *count, int *new_item_id);
-
 int
 db_queue_add_start(struct db_queue_add_info *queue_add_info, int pos);
 
@@ -962,6 +956,9 @@ db_queue_move_byitemid(uint32_t item_id, int pos_to, char shuffle);
 int
 db_queue_move_bypos(int pos_from, int pos_to);
 
+int
+db_queue_move_bypos_range(int range_begin, int range_end, int pos_to);
+
 int
 db_queue_move_byposrelativetoitem(uint32_t from_pos, uint32_t to_offset, uint32_t item_id, char shuffle);
 
@@ -1024,7 +1021,7 @@ int
 db_watch_enum_fetchwd(struct watch_enum *we, uint32_t *wd);
 
 int
-db_backup();
+db_backup(void);
 
 int
 db_perthread_init(void);
@@ -1033,7 +1030,7 @@ void
 db_perthread_deinit(void);
 
 int
-db_init(void);
+db_init(char *sqlite_ext_path);
 
 void
 db_deinit(void);
