@@ -1227,6 +1227,44 @@ static const struct db_upgrade_query db_upgrade_v2200_queries[] =
     { U_v2200_SCVER_MINOR,    "set schema_version_minor to 00" },
   };
 
+/* ---------------------------- 22.00 -> 22.01 ------------------------------ */
+
+#define U_v2201_ALTER_FILES_ADD_LYRICS \
+  "ALTER TABLE files ADD COLUMN lyrics TEXT DEFAULT NULL COLLATE DAAP;"
+
+#define U_v2201_SCVER_MAJOR                    \
+  "UPDATE admin SET value = '22' WHERE key = 'schema_version_major';"
+#define U_v2201_SCVER_MINOR                    \
+  "UPDATE admin SET value = '01' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v2201_queries[] =
+  {
+    { U_v2201_ALTER_FILES_ADD_LYRICS, "alter table files add column lyrics" },
+
+    { U_v2201_SCVER_MAJOR,    "set schema_version_major to 22" },
+    { U_v2201_SCVER_MINOR,    "set schema_version_minor to 01" },
+  };
+
+
+/* ---------------------------- 22.01 -> 22.02 ------------------------------ */
+
+#define U_v2202_ALTER_SPEAKERS_ADD_FORMAT \
+  "ALTER TABLE speakers ADD COLUMN format INTEGER DEFAULT 0;"
+
+#define U_v2202_SCVER_MAJOR                    \
+  "UPDATE admin SET value = '22' WHERE key = 'schema_version_major';"
+#define U_v2202_SCVER_MINOR                    \
+  "UPDATE admin SET value = '02' WHERE key = 'schema_version_minor';"
+
+static const struct db_upgrade_query db_upgrade_v2202_queries[] =
+  {
+    { U_v2202_ALTER_SPEAKERS_ADD_FORMAT, "alter table speakers add column format" },
+
+    { U_v2202_SCVER_MAJOR,    "set schema_version_major to 22" },
+    { U_v2202_SCVER_MINOR,    "set schema_version_minor to 02" },
+  };
+
+
 /* -------------------------- Main upgrade handler -------------------------- */
 
 int
@@ -1437,6 +1475,19 @@ db_upgrade(sqlite3 *hdl, int db_ver)
       if (ret < 0)
 	return -1;
 
+      /* FALLTHROUGH */
+
+    case 2200:
+      ret = db_generic_upgrade(hdl, db_upgrade_v2201_queries, ARRAY_SIZE(db_upgrade_v2201_queries));
+      if (ret < 0)
+	return -1;
+
+      /* FALLTHROUGH */
+
+    case 2201:
+      ret = db_generic_upgrade(hdl, db_upgrade_v2202_queries, ARRAY_SIZE(db_upgrade_v2202_queries));
+      if (ret < 0)
+	return -1;
 
       /* Last case statement is the only one that ends with a break statement! */
       break;
