@@ -63,17 +63,6 @@
 #include "log.h"
 #include "rtsp-internal.h"
 
-// For compability with libevent 2.0 (HAVE_LIBEVENT2_OLD)
-#if defined(_EVENT_HAVE_GETNAMEINFO)
-# define EVENT__HAVE_GETNAMEINFO 1
-#endif
-#if defined(_EVENT_HAVE_GETADDRINFO)
-# define EVENT__HAVE_GETADDRINFO 1
-#endif
-#if defined(_EVENT_HAVE_STRSEP)
-# define EVENT__HAVE_STRSEP 1
-#endif
-
 #ifndef EVENT__HAVE_GETNAMEINFO
 #define NI_MAXSERV 32
 #define NI_MAXHOST 1025
@@ -1702,13 +1691,23 @@ bind_socket_ai(int family, struct addrinfo *ai, int reuse)
         }
 #endif
 
-	if (family == AF_INET6)
-	  setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on));
+	if (family == AF_INET6) {
+		r = setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on));
+		if (r == -1) {
+			event_warn("IPV6_V6ONLY");
+		}
+	}
+
+        r = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on));
+	if (r == -1) {
+		event_warn("SO_KEEPALIVE");
+	}
 
-        setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&on, sizeof(on));
 	if (reuse) {
-		setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
-		    (void *)&on, sizeof(on));
+		r = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on));
+		if (r == -1) {
+			event_warn("SO_REUSEADDR");
+		}
 	}
 
 	if (ai != NULL) {
