@@ -72,25 +72,14 @@ spotify_deinit(void)
 }
 
 int
-spotify_login(const char *username, const char *password, const char **errmsg)
+spotify_login(const char *username, const char *token, const char **errmsg)
 {
   struct spotify_backend *backend = backend_set();
 
   if (!backend || !backend->login)
     return -1;
 
-  return backend->login(username, password, errmsg);
-}
-
-int
-spotify_login_token(const char *username, const char *token, const char **errmsg)
-{
-  struct spotify_backend *backend = backend_set();
-
-  if (!backend || !backend->login_token)
-    return -1;
-
-  return backend->login_token(username, token, errmsg);
+  return backend->login(username, token, errmsg);
 }
 
 void
