@@ -42,10 +42,6 @@
 # include <pthread_np.h>
 #endif
 
-#include <unistr.h>
-#include <unictype.h>
-#include <uninorm.h>
-
 #include <event2/event.h>
 
 #ifdef HAVE_REGEX_H
@@ -67,9 +63,6 @@
 #ifdef LASTFM
 # include "lastfm.h"
 #endif
-#ifdef HAVE_SPOTIFY_H
-# include "spotify.h"
-#endif
 
 
 #define F_SCAN_BULK    (1 << 0)
@@ -83,6 +76,13 @@
 #define F_SCAN_TYPE_AUDIOBOOK    (1 << 2)
 #define F_SCAN_TYPE_COMPILATION  (1 << 3)
 
+#ifdef __linux__
+#define INOTIFY_FLAGS  (IN_ATTRIB | IN_CREATE | IN_DELETE | IN_CLOSE_WRITE | IN_MOVE | IN_DELETE | IN_MOVE_SELF)
+#else
+#define INOTIFY_FLAGS  (IN_CREATE | IN_DELETE | IN_MOVE)
+#endif
+
+
 
 enum file_type {
   FILE_UNKNOWN = 0,
@@ -95,7 +95,6 @@ enum file_type {
   FILE_CTRL_REMOTE,
   FILE_CTRL_RAOP_VERIFICATION,
   FILE_CTRL_LASTFM,
-  FILE_CTRL_SPOTIFY,
   FILE_CTRL_INITSCAN,
   FILE_CTRL_METASCAN, // forced scan for meta, preserves existing db records
   FILE_CTRL_FULLSCAN,
@@ -116,6 +115,7 @@ struct stacked_dir {
 
 static int inofd;
 static struct event *inoev;
+static struct event *mntev;
 static struct deferred_pl *playlists;
 static struct stacked_dir *dirstack;
 
@@ -167,6 +167,20 @@ filescanner_fullrescan();
 
 /* ----------------------- Internal utility functions --------------------- */
 
+static char *
+strip_extension(const char *path)
+{
+  char *ptr;
+  char *result;
+
+  result = strdup(path);
+  ptr = strrchr(result, '.');
+  if (ptr)
+    *ptr = '\0';
+
+  return result;
+}
+
 static int
 virtual_path_make(char *virtual_path, int virtual_path_len, const char *path)
 {
@@ -174,10 +188,10 @@ virtual_path_make(char *virtual_path, int virtual_path_len, const char *path)
 
   ret = snprintf(virtual_path, virtual_path_len, "/file:%s", path);
   if ((ret < 0) || (ret >= virtual_path_len))
-  {
-    DPRINTF(E_LOG, L_SCAN, "Virtual path '/file:%s', virtual_path_len exceeded (%d/%d)\n", path, ret, virtual_path_len);
-    return -1;
-  }
+    {
+      DPRINTF(E_LOG, L_SCAN, "Virtual path '/file:%s', virtual_path_len exceeded (%d/%d)\n", path, ret, virtual_path_len);
+      return -1;
+    }
 
   return 0;
 }
@@ -326,7 +340,7 @@ file_type_get(const char *path) {
   if (file_type_ignore(ext))
     return FILE_IGNORE;
 
-  if ((strcasecmp(ext, ".m3u") == 0) || (strcasecmp(ext, ".pls") == 0))
+  if ((strcasecmp(ext, ".m3u") == 0) || (strcasecmp(ext, ".pls") == 0) || (strcasecmp(ext, ".m3u8") == 0))
     return FILE_PLAYLIST;
 
   if (strcasecmp(ext, ".smartpl") == 0)
@@ -350,9 +364,6 @@ file_type_get(const char *path) {
   if (strcasecmp(ext, ".lastfm") == 0)
     return FILE_CTRL_LASTFM;
 
-  if (strcasecmp(ext, ".spotify") == 0)
-    return FILE_CTRL_SPOTIFY;
-
   if (strcasecmp(ext, ".init-rescan") == 0)
     return FILE_CTRL_INITSCAN;
 
@@ -392,17 +403,13 @@ filename_from_path(const char *path)
 }
 
 char *
-strip_extension(const char *path)
+title_from_path(const char *path)
 {
-  char *ptr;
-  char *result;
+  const char *filename;
 
-  result = strdup(path);
-  ptr = strrchr(result, '.');
-  if (ptr)
-    *ptr = '\0';
+  filename = filename_from_path(path);
 
-  return result;
+  return strip_extension(filename);
 }
 
 int
@@ -429,12 +436,9 @@ parent_dir(const char **current, const char *path)
 int
 playlist_fill(struct playlist_info *pli, const char *path)
 {
-  const char *filename;
   char virtual_path[PATH_MAX];
   int ret;
 
-  filename = filename_from_path(path);
-
   ret = virtual_path_make(virtual_path, sizeof(virtual_path), path);
   if (ret < 0)
     return -1;
@@ -443,8 +447,9 @@ playlist_fill(struct playlist_info *pli, const char *path)
 
   pli->type  = PL_PLAIN;
   pli->path  = strdup(path);
-  pli->title = strip_extension(filename); // Will alloc
+  pli->title = title_from_path(path); // Will alloc
   pli->virtual_path = strip_extension(virtual_path); // Will alloc
+  pli->scan_kind = SCAN_KIND_FILES;
 
   pli->directory_id = get_parent_dir_id(path);
 
@@ -568,10 +573,12 @@ process_regular_file(const char *file, struct stat *sb, int type, int flags, int
   char virtual_path[PATH_MAX];
   int ret;
 
-  // Will return 0 if file is not in library or if file mtime is newer than library timestamp
-  // - note if mtime is 0 then we always scan the file
   if (!(flags & F_SCAN_METARESCAN))
     {
+      // Will return 0 if file is not in library or if file mtime is not older
+      // than the library timestamp. If mtime is equal we must rescan, since a
+      // fast update may have been made, see issue #1782. If mtime is 0 then we
+      // always scan.
       ret = db_file_ping_bypath(file, sb->st_mtime);
       if ((sb->st_mtime != 0) && (ret != 0))
         return;
@@ -593,6 +600,7 @@ process_regular_file(const char *file, struct stat *sb, int type, int flags, int
   mfi.virtual_path = strdup(virtual_path);
 
   mfi.directory_id = dir_id;
+  mfi.scan_kind = SCAN_KIND_FILES;
 
   if (S_ISFIFO(sb->st_mode))
     {
@@ -700,24 +708,13 @@ process_file(char *file, struct stat *sb, enum file_type file_type, int scan_typ
 #endif
 	break;
 
-      case FILE_CTRL_SPOTIFY:
-#ifdef HAVE_SPOTIFY_H
-	if (flags & F_SCAN_BULK)
-	  DPRINTF(E_LOG, L_SCAN, "Bulk scan will ignore '%s' (to process, add it after startup)\n", file);
-	else
-	  kickoff(spotify_login, file, 2);
-#else
-	DPRINTF(E_LOG, L_SCAN, "Found '%s', but this version was built without Spotify support\n", file);
-#endif
-	break;
-
       case FILE_CTRL_INITSCAN:
 	if (flags & F_SCAN_BULK)
 	  break;
 
 	DPRINTF(E_LOG, L_SCAN, "Startup rescan triggered, found init-rescan file: %s\n", file);
 
-	library_rescan();
+	library_rescan(0);
 	break;
 
       case FILE_CTRL_METASCAN:
@@ -726,7 +723,7 @@ process_file(char *file, struct stat *sb, enum file_type file_type, int scan_typ
 
 	DPRINTF(E_LOG, L_SCAN, "Meta rescan triggered, found meta-rescan file: %s\n", file);
 
-	library_metarescan();
+	library_metarescan(0);
 	break;
 
       case FILE_CTRL_FULLSCAN:
@@ -834,7 +831,6 @@ process_directory(char *path, int parent_id, int flags)
   if (!dirp)
     {
       DPRINTF(E_LOG, L_SCAN, "Could not open directory %s: %s\n", path, strerror(errno));
-
       return;
     }
 
@@ -842,9 +838,12 @@ process_directory(char *path, int parent_id, int flags)
 
   ret = virtual_path_make(virtual_path, sizeof(virtual_path), path);
   if (ret < 0)
-    return;
+    {
+      closedir(dirp);
+      return;
+    }
 
-  dir_id = db_directory_addorupdate(virtual_path, path, 0, parent_id);
+  dir_id = library_directory_save(virtual_path, path, 0, parent_id, SCAN_KIND_FILES);
   if (dir_id <= 0)
     {
       DPRINTF(E_LOG, L_SCAN, "Insert or update of directory failed '%s'\n", virtual_path);
@@ -871,7 +870,6 @@ process_directory(char *path, int parent_id, int flags)
       if (errno)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "readdir error in %s: %s\n", path, strerror(errno));
-
 	  break;
 	}
 
@@ -897,7 +895,6 @@ process_directory(char *path, int parent_id, int flags)
       if (ret < 0)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Skipping %s, read_attributes() failed\n", entry);
-
 	  continue;
 	}
 
@@ -926,15 +923,10 @@ process_directory(char *path, int parent_id, int flags)
 
   // Add inotify watch (for FreeBSD we limit the flags so only dirs will be
   // opened, otherwise we will be opening way too many files)
-#ifdef __linux__
-  wi.wd = inotify_add_watch(inofd, path, IN_ATTRIB | IN_CREATE | IN_DELETE | IN_CLOSE_WRITE | IN_MOVE | IN_DELETE | IN_MOVE_SELF);
-#else
-  wi.wd = inotify_add_watch(inofd, path, IN_CREATE | IN_DELETE | IN_MOVE);
-#endif
+  wi.wd = inotify_add_watch(inofd, path, INOTIFY_FLAGS);
   if (wi.wd < 0)
     {
       DPRINTF(E_WARN, L_SCAN, "Could not create inotify watch for %s: %s\n", path, strerror(errno));
-
       return;
     }
 
@@ -975,7 +967,7 @@ process_parent_directories(char *path)
       if (ret < 0)
 	return 0;
 
-      dir_id = db_directory_addorupdate(virtual_path, buf, 0, dir_id);
+      dir_id = library_directory_save(virtual_path, buf, 0, dir_id, SCAN_KIND_FILES);
       if (dir_id <= 0)
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Insert or update of directory failed '%s'\n", virtual_path);
@@ -1130,8 +1122,8 @@ static void
 process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 {
   struct watch_enum we;
+  struct watch_info dummy_wi;
   uint32_t rm_wd;
-  char *s;
   int flags = 0;
   int ret;
   int parent_id;
@@ -1139,13 +1131,6 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 
   DPRINTF(E_DBG, L_SCAN, "Directory event: 0x%08x, cookie 0x%08x, wd %d\n", ie->mask, ie->cookie, wi->wd);
 
-  if (ie->mask & IN_UNMOUNT)
-    {
-      db_file_disable_bymatch(path, STRIP_NONE, 0);
-      db_pl_disable_bymatch(path, STRIP_NONE, 0);
-      db_directory_disable_bymatch(path, STRIP_NONE, 0);
-    }
-
   if (ie->mask & IN_MOVE_SELF)
     {
       /* A directory we know about, that got moved from a place
@@ -1222,12 +1207,9 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
       DPRINTF(E_DBG, L_SCAN, "Directory permissions changed (%s): %s\n", wi->path, path);
 
       // Find out if we are already watching the dir (ret will be 0)
-      s = wi->path;
-      wi->path = path;
-      ret = db_watch_get_bypath(wi);
+      ret = db_watch_get_bypath(&dummy_wi, path);
       if (ret == 0)
-	free(wi->path);
-      wi->path = s;
+	free_wi(&dummy_wi, 1);
 
       // We don't use access() or euidaccess() because they don't work with ACL's
       // - this also means we can't check for executable permission, which stat()
@@ -1255,7 +1237,7 @@ process_inotify_dir(struct watch_info *wi, char *path, struct inotify_event *ie)
 	  DPRINTF(E_INFO, L_SCAN, "Directory event, but '%s' already being watched\n", path);
 	}
 
-      if (fd > 0)
+      if (fd >= 0)
 	close(fd);
     }
 
@@ -1339,7 +1321,7 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	  ie->mask |= IN_CLOSE_WRITE;
 	}
 
-      if (fd > 0)
+      if (fd >= 0)
 	close(fd);
     }
 
@@ -1347,6 +1329,12 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
     {
       DPRINTF(E_DBG, L_SCAN, "File moved to: %s\n", path);
 
+      /* handle overwriting an existing file, no inotify event generated for the
+       * overwrite on existing file before we update the path of moved file
+       */
+      db_file_delete_bypath(path);
+      cache_artwork_delete_by_path(path);
+
       ret = db_file_enable_bycookie(ie->cookie, path, filename_from_path(path));
 
       if (ret > 0)
@@ -1354,7 +1342,7 @@ process_inotify_file(struct watch_info *wi, char *path, struct inotify_event *ie
 	  ret = virtual_path_make(dir_vpath, sizeof(dir_vpath), path);
 	  if (ret >= 0)
 	    {
-	      ptr = strrchr(dir_vpath, '/');
+	      CHECK_NULL(L_SCAN, ptr = strrchr(dir_vpath, '/'));
 	      *ptr = '\0';
 
 	      dir_id = db_directory_id_byvirtualpath(dir_vpath);
@@ -1514,6 +1502,7 @@ inotify_cb(int fd, short event, void *arg)
 {
   struct inotify_event *ie;
   struct watch_info wi;
+  struct stat sb;
   uint8_t *buf;
   uint8_t *ptr;
   char path[PATH_MAX];
@@ -1557,8 +1546,7 @@ inotify_cb(int fd, short event, void *arg)
        * the memory space for ie[1+] contains the name of the file
        * see the inotify documentation
        */
-      wi.wd = ie->wd;
-      ret = db_watch_get_bywd(&wi);
+      ret = db_watch_get_bywd(&wi, ie->wd);
       if (ret < 0)
 	{
 	  if (!(ie->mask & IN_IGNORED))
@@ -1572,30 +1560,38 @@ inotify_cb(int fd, short event, void *arg)
 	  DPRINTF(E_DBG, L_SCAN, "%s deleted or backing filesystem unmounted!\n", wi.path);
 
 	  db_watch_delete_bywd(ie->wd);
-	  free(wi.path);
-	  continue;
+
+	  // Is the directory gone?
+	  if (! (lstat(wi.path, &sb) == 0 && S_ISDIR(sb.st_mode)))
+	    {
+	      free_wi(&wi, 1);
+	      continue;
+	    }
+
+	  // After an unmount event the mount point is a regular dir that we must process
+	  ie->mask |= IN_CREATE;
 	}
 
       path[0] = '\0';
 
-      ret = snprintf(path, PATH_MAX, "%s", wi.path);
-      if ((ret < 0) || (ret >= PATH_MAX))
+      ret = snprintf(path, sizeof(path), "%s", wi.path);
+      if ((ret < 0) || (ret >= sizeof(path)))
 	{
 	  DPRINTF(E_LOG, L_SCAN, "Skipping event under %s, PATH_MAX exceeded\n", wi.path);
 
-	  free(wi.path);
+	  free_wi(&wi, 1);
 	  continue;
 	}
 
       if (ie->len > 0)
 	{
-	  namelen = PATH_MAX - ret;
+	  namelen = sizeof(path) - ret;
 	  ret = snprintf(path + ret, namelen, "/%s", ie->name);
 	  if ((ret < 0) || (ret >= namelen))
 	    {
 	      DPRINTF(E_LOG, L_SCAN, "Skipping %s/%s, PATH_MAX exceeded\n", wi.path, ie->name);
 
-	      free(wi.path);
+	      free_wi(&wi, 1);
 	      continue;
 	    }
 	}
@@ -1613,7 +1609,7 @@ inotify_cb(int fd, short event, void *arg)
 #else
 	process_inotify_file_defer(&wi, path, ie);
 #endif
-      free(wi.path);
+      free_wi(&wi, 1);
     }
 
   free(buf);
@@ -1621,28 +1617,77 @@ inotify_cb(int fd, short event, void *arg)
   event_add(inoev, NULL);
 }
 
+static void
+mount_cb(int fd, short event, void *arg)
+{
+  struct watch_info wi = { 0 };
+  char *path = NULL;
+  enum mountwatch_event mev;
+  int parent_id;
+  int ret;
+
+  mev = mountwatch_event_get(&path);
+  if (mev == MOUNTWATCH_ERR || mev == MOUNTWATCH_NONE)
+    goto readd;
+
+  // Check if this is a location we are watching
+  ret = db_watch_get_bypath(&wi, path);
+  if (ret < 0)
+    goto readd;
+
+  if (mev == MOUNTWATCH_MOUNT)
+    {
+      DPRINTF(E_DBG, L_SCAN, "MNT_MOUNT path %s\n", path);
+
+      // After a mount, the inotify watch we had on the mount point will no
+      // longer be working, so remove.
+      inotify_rm_watch(inofd, wi.wd);
+      db_watch_delete_bywd(wi.wd);
+
+      // Adds watches, scans and sets disabled = 0
+      parent_id = get_parent_dir_id(path);
+      process_directories(path, parent_id, 0);
+    }
+  else if (mev == MOUNTWATCH_UNMOUNT)
+    {
+      DPRINTF(E_DBG, L_SCAN, "MNT_UNMOUNT path %s\n", path);
+
+      db_file_disable_bymatch(path, STRIP_NONE, 0);
+      db_pl_disable_bymatch(path, STRIP_NONE, 0);
+      db_directory_disable_bymatch(path, STRIP_NONE, 0);
+    }
+
+ readd:
+  free_wi(&wi, 1);
+  free(path);
+  event_add(mntev, NULL);
+}
+
 /* Thread: main & scan */
 static int
 inofd_event_set(void)
 {
+  int mntfd;
+
   inofd = inotify_init1(IN_CLOEXEC);
   if (inofd < 0)
     {
       DPRINTF(E_FATAL, L_SCAN, "Could not create inotify fd: %s\n", strerror(errno));
-
       return -1;
     }
+  else
+    CHECK_NULL(L_SCAN, inoev = event_new(evbase_lib, inofd, EV_READ, inotify_cb, NULL));
 
-  inoev = event_new(evbase_lib, inofd, EV_READ, inotify_cb, NULL);
+  // If mountwatch cannot be initialized we create mntev as a timer (that will
+  // never trigger), so that all the event_add() don't need a "if (mntev)" test.
+  mntfd = mountwatch_init();
+  if (mntfd >= 0)
+    CHECK_NULL(L_SCAN, mntev = event_new(evbase_lib, mntfd, EV_READ, mount_cb, NULL));
+  else
+    CHECK_NULL(L_SCAN, mntev = evtimer_new(evbase_lib, mount_cb, NULL));
 
 #ifndef __linux__
-  deferred_inoev = evtimer_new(evbase_lib, inotify_deferred_cb, NULL);
-  if (!deferred_inoev)
-    {
-      DPRINTF(E_LOG, L_SCAN, "Could not create deferred inotify event\n");
-
-      return -1;
-    }
+  CHECK_NULL(L_SCAN, deferred_inoev = evtimer_new(evbase_lib, inotify_deferred_cb, NULL));
 #endif
 
   return 0;
@@ -1655,6 +1700,9 @@ inofd_event_unset(void)
 #ifndef __linux__
   event_free(deferred_inoev);
 #endif
+  event_free(mntev);
+  mountwatch_deinit();
+
   event_free(inoev);
   close(inofd);
 }
@@ -1681,6 +1729,7 @@ filescanner_initscan()
     {
       /* Enable inotify */
       event_add(inoev, NULL);
+      event_add(mntev, NULL);
     }
   return 0;
 }
@@ -1699,6 +1748,7 @@ filescanner_rescan()
     {
       /* Enable inotify */
       event_add(inoev, NULL);
+      event_add(mntev, NULL);
     }
   return 0;
 }
@@ -1717,6 +1767,7 @@ filescanner_metarescan()
     {
       /* Enable inotify */
       event_add(inoev, NULL);
+      event_add(mntev, NULL);
     }
   return 0;
 }
@@ -1734,54 +1785,114 @@ filescanner_fullrescan()
     {
       /* Enable inotify */
       event_add(inoev, NULL);
+      event_add(mntev, NULL);
     }
   return 0;
 }
 
+static int
+filescanner_write_metadata(struct media_file_info *mfi)
+{
+  return write_metadata_ffmpeg(mfi);
+}
+
+static int
+queue_item_file_add(const char *sub_uri, int position, char reshuffle, uint32_t item_id, int *count, int *new_item_id)
+{
+  struct query_params query_params = { 0 };
+  int64_t id;
+
+  if (strncmp(sub_uri, "artist:", strlen("artist:")) == 0)
+    {
+      if (safe_atoi64(sub_uri + (strlen("artist:")), &id) < 0)
+	return -1;
+
+      query_params.type = Q_GROUP_ITEMS;
+      query_params.sort = S_ALBUM;
+      query_params.persistentid = id;
+    }
+  else if (strncmp(sub_uri, "album:", strlen("album:")) == 0)
+    {
+      if (safe_atoi64(sub_uri + (strlen("album:")), &id) < 0)
+	return -1;
+
+      query_params.type = Q_GROUP_ITEMS;
+      query_params.sort = S_ALBUM;
+      query_params.persistentid = id;
+    }
+  else if (strncmp(sub_uri, "track:", strlen("track:")) == 0)
+    {
+      if (safe_atoi64(sub_uri + (strlen("track:")), &id) < 0)
+	return -1;
+
+      query_params.type = Q_ITEMS;
+      query_params.id = id;
+    }
+  else if (strncmp(sub_uri, "playlist:", strlen("playlist:")) == 0)
+    {
+      if (safe_atoi64(sub_uri + (strlen("playlist:")), &id) < 0)
+	return -1;
+
+      query_params.type = Q_PLITEMS;
+      query_params.id = id;
+    }
+  else
+    {
+      return -1;
+    }
+
+  return db_queue_add_by_query(&query_params, reshuffle, item_id, position, count, new_item_id);
+}
+
 static int
 queue_item_stream_add(const char *path, int position, char reshuffle, uint32_t item_id, int *count, int *new_item_id)
 {
-  struct media_file_info mfi;
+  struct media_file_info mfi = { 0 };
   struct db_queue_item qi;
   struct db_queue_add_info queue_add_info;
   int ret;
 
-  memset(&mfi, 0, sizeof(struct media_file_info));
-
   scan_metadata_stream(&mfi, path);
 
   db_queue_item_from_mfi(&qi, &mfi);
 
   ret = db_queue_add_start(&queue_add_info, position);
-  if (ret == 0)
-    {
-      ret = db_queue_add_next(&queue_add_info, &qi);
-      ret = db_queue_add_end(&queue_add_info, reshuffle, item_id, ret);
-      if (ret == 0)
-	{
-	  if (count)
-	    *count = queue_add_info.count;
-	  if (new_item_id)
-	    *new_item_id = queue_add_info.new_item_id;
-	}
-    }
+  if (ret < 0)
+    goto error;
+
+  ret = db_queue_add_next(&queue_add_info, &qi);
+  ret = db_queue_add_end(&queue_add_info, reshuffle, item_id, ret);
+  if (ret < 0)
+    goto error;
+
+  if (count)
+    *count = queue_add_info.count;
+  if (new_item_id)
+    *new_item_id = queue_add_info.new_item_id;
 
   free_queue_item(&qi, 1);
   free_mfi(&mfi, 1);
-
   return 0;
+
+ error:
+  free_queue_item(&qi, 1);
+  free_mfi(&mfi, 1);
+  return -1;
 }
 
 static int
 queue_item_add(const char *uri, int position, char reshuffle, uint32_t item_id, int *count, int *new_item_id)
 {
-  if (net_is_http_or_https(uri))
-    {
-      queue_item_stream_add(uri, position, reshuffle, item_id, count, new_item_id);
-      return LIBRARY_OK;
-    }
+  int ret;
 
-  return LIBRARY_PATH_INVALID;
+  if (strncmp(uri, "library:", strlen("library:")) == 0)
+    ret = queue_item_file_add(uri + strlen("library:"), position, reshuffle, item_id, count, new_item_id);
+  else if (net_is_http_or_https(uri))
+    ret = queue_item_stream_add(uri, position, reshuffle, item_id, count, new_item_id);
+  else
+    ret = -1;
+
+  return (ret == 0) ? LIBRARY_OK : LIBRARY_PATH_INVALID;
 }
 
 static const char *
@@ -1808,7 +1919,7 @@ check_path_in_directories(const char *path)
   bool ret;
 
   if (strstr(path, "/../"))
-    return NULL;
+    return false;
 
   tmp_path = strdup(path);
   dir = dirname(tmp_path);
@@ -1925,7 +2036,7 @@ playlist_add_files(FILE *fp, int pl_id, const char *virtual_path)
 
   if (qp.results > 0)
     {
-      while (((ret = db_query_fetch_file(&qp, &dbmfi)) == 0) && (dbmfi.id))
+      while ((ret = db_query_fetch_file(&dbmfi, &qp)) == 0)
         {
 	  if ((safe_atou32(dbmfi.data_kind, &data_kind) < 0)
 	      || (data_kind == DATA_KIND_PIPE))
@@ -2158,9 +2269,7 @@ filescanner_init(void)
 
   ret = inofd_event_set();
   if (ret < 0)
-    {
-      return -1;
-    }
+    return -1;
 
   return 0;
 }
@@ -2175,7 +2284,7 @@ filescanner_deinit(void)
 
 struct library_source filescanner =
 {
-  .name = "filescanner",
+  .scan_kind = SCAN_KIND_FILES,
   .disabled = 0,
   .init = filescanner_init,
   .deinit = filescanner_deinit,
@@ -2183,6 +2292,7 @@ struct library_source filescanner =
   .rescan = filescanner_rescan,
   .metarescan = filescanner_metarescan,
   .fullrescan = filescanner_fullrescan,
+  .write_metadata = filescanner_write_metadata,
   .playlist_item_add = playlist_item_add,
   .playlist_remove = playlist_remove,
   .queue_save = queue_save,
