@@ -22,10 +22,20 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
 
-#include <stdint.h>
+// For fstat()
+#include <sys/types.h>
+#include <sys/stat.h>
+
+// For file copy (fcopyfile currently Mac OSX only)
+#include <fcntl.h>
+#if defined(HAVE_FCOPYFILE)
+#include <copyfile.h>
+#endif
 
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
@@ -37,13 +47,19 @@
 #include "http.h"
 #include "conffile.h"
 
+// From libavutil 57.37.100
+#if !defined(HAVE_DECL_AV_DICT_ITERATE) || !(HAVE_DECL_AV_DICT_ITERATE)
+# define av_dict_iterate(dict, entry) av_dict_get((dict), "", (entry), AV_DICT_IGNORE_SUFFIX)
+#endif
+
 /* Mapping between the metadata name(s) and the offset
  * of the equivalent metadata field in struct media_file_info */
 struct metadata_map {
   char *key;
   int as_int;
   size_t offset;
-  int (*handler_function)(struct media_file_info *, char *);
+  int (*handler_function)(struct media_file_info *, const char *);
+  int flags;
 };
 
 // Used for passing errors to DPRINTF (can't count on av_err2str being present)
@@ -57,7 +73,7 @@ err2str(int errnum)
 }
 
 static int
-parse_genre(struct media_file_info *mfi, char *genre_string)
+parse_genre(struct media_file_info *mfi, const char *genre_string)
 {
   char **genre = (char**)((char *) mfi + mfi_offsetof(genre));
   char *ptr;
@@ -78,12 +94,17 @@ parse_genre(struct media_file_info *mfi, char *genre_string)
 }
 
 static int
-parse_slash_separated_ints(char *string, uint32_t *firstval, uint32_t *secondval)
+parse_slash_separated_ints(const char *string, uint32_t *firstval, uint32_t *secondval)
 {
   int numvals = 0;
+  char buf[64];
   char *ptr;
 
-  ptr = strchr(string, '/');
+  // dict.h: "The returned entry key or value must not be changed, or it will
+  // cause undefined behavior" -> so we must make a copy
+  snprintf(buf, sizeof(buf), "%s", string);
+
+  ptr = strchr(buf, '/');
   if (ptr)
     {
       *ptr = '\0';
@@ -91,14 +112,14 @@ parse_slash_separated_ints(char *string, uint32_t *firstval, uint32_t *secondval
         numvals++;
     }
 
-  if (safe_atou32(string, firstval) == 0)
+  if (safe_atou32(buf, firstval) == 0)
     numvals++;
 
   return numvals;
 }
 
 static int
-parse_track(struct media_file_info *mfi, char *track_string)
+parse_track(struct media_file_info *mfi, const char *track_string)
 {
   uint32_t *track = (uint32_t *) ((char *) mfi + mfi_offsetof(track));
   uint32_t *total_tracks = (uint32_t *) ((char *) mfi + mfi_offsetof(total_tracks));
@@ -107,7 +128,7 @@ parse_track(struct media_file_info *mfi, char *track_string)
 }
 
 static int
-parse_disc(struct media_file_info *mfi, char *disc_string)
+parse_disc(struct media_file_info *mfi, const char *disc_string)
 {
   uint32_t *disc = (uint32_t *) ((char *) mfi + mfi_offsetof(disc));
   uint32_t *total_discs = (uint32_t *) ((char *) mfi + mfi_offsetof(total_discs));
@@ -116,7 +137,7 @@ parse_disc(struct media_file_info *mfi, char *disc_string)
 }
 
 static int
-parse_date(struct media_file_info *mfi, char *date_string)
+parse_date(struct media_file_info *mfi, const char *date_string)
 {
   char year_string[32];
   uint32_t *year = (uint32_t *) ((char *) mfi + mfi_offsetof(year));
@@ -128,10 +149,11 @@ parse_date(struct media_file_info *mfi, char *date_string)
   if ((*year == 0) && (safe_atou32(date_string, year) == 0))
     ret++;
 
-  if ( strptime(date_string, "%FT%T%z", &tm) // ISO 8601, %F=%Y-%m-%d, %T=%H:%M:%S
-       || strptime(date_string, "%F %T", &tm)
-       || strptime(date_string, "%F %H:%M", &tm)
-       || strptime(date_string, "%F", &tm)
+  // musl doesn't support %F, so %Y-%m-%d is used instead
+  if ( strptime(date_string, "%Y-%m-%dT%T%z", &tm) // ISO 8601, %T=%H:%M:%S
+       || strptime(date_string, "%Y-%m-%d %T", &tm)
+       || strptime(date_string, "%Y-%m-%d %H:%M", &tm)
+       || strptime(date_string, "%Y-%m-%d", &tm)
      )
     {
       *date_released = mktime(&tm);
@@ -141,7 +163,7 @@ parse_date(struct media_file_info *mfi, char *date_string)
   if ((*date_released == 0) && (*year != 0))
     {
       snprintf(year_string, sizeof(year_string), "%" PRIu32 "-01-01T12:00:00", *year);
-      if (strptime(year_string, "%FT%T", &tm))
+      if (strptime(year_string, "%Y-%m-%dT%T", &tm))
 	{
 	  *date_released = mktime(&tm);
 	  ret++;
@@ -152,7 +174,7 @@ parse_date(struct media_file_info *mfi, char *date_string)
 }
 
 static int
-parse_albumid(struct media_file_info *mfi, char *id_string)
+parse_albumid(struct media_file_info *mfi, const char *id_string)
 {
   // Already set by a previous tag that we give higher priority
   if (mfi->songalbumid)
@@ -163,6 +185,28 @@ parse_albumid(struct media_file_info *mfi, char *id_string)
   return 1;
 }
 
+static int
+parse_rating(struct media_file_info *mfi, const char *rating_string)
+{
+  cfg_t *library = cfg_getsec(cfg, "library");
+  int max_rating;
+
+  if (!cfg_getbool(library, "read_rating"))
+    return 0;
+
+  if (safe_atou32(rating_string, &mfi->rating) < 0)
+    return 0;
+
+  // Make sure mfi->rating is in proper range
+  max_rating = cfg_getint(library, "max_rating");
+  if (max_rating < 5) // Invalid config
+    max_rating = DB_FILES_RATING_MAX;
+
+  mfi->rating = MIN(DB_FILES_RATING_MAX * mfi->rating / max_rating, DB_FILES_RATING_MAX);
+  return 1;
+}
+
+
 /* Lookup is case-insensitive, first occurrence takes precedence */
 static const struct metadata_map md_map_generic[] =
   {
@@ -186,6 +230,8 @@ static const struct metadata_map md_map_generic[] =
     { "artist-sort",  0, mfi_offsetof(artist_sort),        NULL },
     { "album-sort",   0, mfi_offsetof(album_sort),         NULL },
     { "compilation",  1, mfi_offsetof(compilation),        NULL },
+    { "lyrics",       0, mfi_offsetof(lyrics),             NULL,       AV_DICT_IGNORE_SUFFIX },
+    { "rating",       1, mfi_offsetof(rating),             parse_rating },
 
     // ALAC sort tags
     { "sort_name",           0, mfi_offsetof(title_sort),         NULL },
@@ -277,34 +323,27 @@ static const struct metadata_map md_map_id3[] =
     { NULL,                  0, 0,                                   NULL }
   };
 
-
 static int
-extract_metadata_core(struct media_file_info *mfi, AVDictionary *md, const struct metadata_map *md_map)
+extract_metadata_from_dict(struct media_file_info *mfi, AVDictionary *md, const struct metadata_map *md_map)
 {
   AVDictionaryEntry *mdt;
   char **strval;
   uint32_t *intval;
-  int mdcount;
+  int mdcount = 0;
   int i;
-  int ret;
 
 #if 0
   /* Dump all the metadata reported by ffmpeg */
-  mdt = NULL;
-  while ((mdt = av_dict_get(md, "", mdt, AV_DICT_IGNORE_SUFFIX)) != NULL)
-    DPRINTF(E_DBG, L_SCAN, " -> %s = %s\n", mdt->key, mdt->value);
+  for (mdt = av_dict_iterate(md, NULL); mdt; mdt = av_dict_iterate(md, mdt))
+    {
+      DPRINTF(E_DBG, L_SCAN, " -> %s = %s\n", mdt->key, mdt->value);
+    }
 #endif
 
-  mdcount = 0;
-
-  /* Extract actual metadata */
   for (i = 0; md_map[i].key != NULL; i++)
     {
-      mdt = av_dict_get(md, md_map[i].key, NULL, 0);
-      if (mdt == NULL)
-	continue;
-
-      if ((mdt->value == NULL) || (strlen(mdt->value) == 0))
+      mdt = av_dict_get(md, md_map[i].key, NULL, md_map[i].flags);
+      if (!mdt || !mdt->value || strlen(mdt->value) == 0)
 	continue;
 
       if (md_map[i].handler_function)
@@ -313,26 +352,27 @@ extract_metadata_core(struct media_file_info *mfi, AVDictionary *md, const struc
 	  continue;
 	}
 
-      mdcount++;
-
       if (!md_map[i].as_int)
 	{
 	  strval = (char **) ((char *) mfi + md_map[i].offset);
 
-	  if (*strval == NULL)
-	    *strval = strdup(mdt->value);
+	  if (*strval != NULL)
+	    continue;
+
+	  *strval = strdup(mdt->value);
 	}
       else
 	{
 	  intval = (uint32_t *) ((char *) mfi + md_map[i].offset);
 
-	  if (*intval == 0)
-	    {
-	      ret = safe_atou32(mdt->value, intval);
-	      if (ret < 0)
-		continue;
-	    }
+	  if (*intval != 0)
+	    continue;
+
+	  if (safe_atou32(mdt->value, intval) < 0)
+	    continue;
 	}
+
+      mdcount++;
     }
 
   return mdcount;
@@ -341,14 +381,12 @@ extract_metadata_core(struct media_file_info *mfi, AVDictionary *md, const struc
 static int
 extract_metadata(struct media_file_info *mfi, AVFormatContext *ctx, AVStream *audio_stream, AVStream *video_stream, const struct metadata_map *md_map)
 {
-  int mdcount;
+  int mdcount = 0;
   int ret;
 
-  mdcount = 0;
-
   if (ctx->metadata)
     {
-      ret = extract_metadata_core(mfi, ctx->metadata, md_map);
+      ret = extract_metadata_from_dict(mfi, ctx->metadata, md_map);
       mdcount += ret;
 
       DPRINTF(E_DBG, L_SCAN, "Picked up %d tags from file metadata\n", ret);
@@ -356,7 +394,7 @@ extract_metadata(struct media_file_info *mfi, AVFormatContext *ctx, AVStream *au
 
   if (audio_stream->metadata)
     {
-      ret = extract_metadata_core(mfi, audio_stream->metadata, md_map);
+      ret = extract_metadata_from_dict(mfi, audio_stream->metadata, md_map);
       mdcount += ret;
 
       DPRINTF(E_DBG, L_SCAN, "Picked up %d tags from audio stream metadata\n", ret);
@@ -364,7 +402,7 @@ extract_metadata(struct media_file_info *mfi, AVFormatContext *ctx, AVStream *au
 
   if (video_stream && video_stream->metadata)
     {
-      ret = extract_metadata_core(mfi, video_stream->metadata, md_map);
+      ret = extract_metadata_from_dict(mfi, video_stream->metadata, md_map);
       mdcount += ret;
 
       DPRINTF(E_DBG, L_SCAN, "Picked up %d tags from video stream metadata\n", ret);
@@ -518,7 +556,7 @@ scan_metadata_ffmpeg(struct media_file_info *mfi, const char *file)
 		if (mfi->bits_per_sample == 0)
 		  mfi->bits_per_sample = av_get_bits_per_sample(codec_id);
 		mfi->channels = channels;
-	      } 
+	      }
 	    break;
 
 	  default:
@@ -764,3 +802,372 @@ scan_metadata_ffmpeg(struct media_file_info *mfi, const char *file)
 
   return 0;
 }
+
+
+/* ----------------------- Writing metadata to files ------------------------ */
+
+static int
+fast_copy(int fd_dst, int fd_src)
+{
+  // Here we use kernel-space copying for performance reasons
+#if defined(HAVE_FCOPYFILE)
+  return fcopyfile(fd_src, fd_dst, 0, COPYFILE_ALL);
+#elif defined(HAVE_COPY_FILE_RANGE)
+  struct stat fileinfo = { 0 };
+  ssize_t bytes_copied;
+
+  fstat(fd_src, &fileinfo);
+  bytes_copied = copy_file_range(fd_src, NULL, fd_dst, NULL, fileinfo.st_size, 0);
+  if (bytes_copied < 0 || bytes_copied != fileinfo.st_size)
+    return -1;
+
+  return 0;
+#else
+  unsigned char buf[4096];
+  ssize_t nr;
+
+  while (1)
+    {
+      nr = read(fd_src, buf, sizeof(buf));
+      if (nr == -1)
+	return -1;
+      else if (nr == 0)
+	return 0;
+
+      if (write(fd_dst, buf, nr) != nr)
+	return -1;
+    }
+
+  return 0;
+#endif
+}
+
+static int
+file_copy(const char *dst, const char *src)
+{
+  int fd_src = -1;
+  int fd_dst = -1;
+  int ret;
+
+  fd_src = open(src, O_RDONLY);
+  if (fd_src < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error opening source '%s' for copy: %s\n", src, strerror(errno));
+      goto error;
+    }
+
+  fd_dst = open(dst, O_WRONLY);
+  if (fd_src < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error opening destination '%s' for copy: %s\n", dst, strerror(errno));
+      goto error;
+    }
+
+  ret = fast_copy(fd_dst, fd_src);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error copying '%s' to file '%s': %s\n", src, dst, strerror(errno));
+      goto error;
+    }
+
+  close(fd_src);
+  close(fd_dst);
+  return 0;
+
+ error:
+  if (fd_src != -1)
+    close(fd_src);
+  if (fd_dst != -1)
+    close(fd_dst);
+  return -1;
+}
+
+static int
+file_copy_to_tmp(char *dst, size_t dst_size, const char *src)
+{
+  int fd_src = -1;
+  int fd_dst = -1;
+  const char *ext;
+  int ret;
+
+  ext = strrchr(src, '.');
+  if (!ext || strlen(ext) < 2)
+    return -1;
+
+  // Obviously, copying only requires read access, but we will need write access
+  // later, so let's fail early if it isn't going to work.
+  fd_src = open(src, O_RDWR);
+  if (fd_src < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error opening '%s' for metadata update: %s\n", src, strerror(errno));
+      goto error;
+    }
+
+  ret = snprintf(dst, dst_size, "/tmp/owntone.tmpXXXXXX%s", ext);
+  if (ret < 0 || ret >= dst_size)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error creating tmp file name\n");
+      goto error;
+    }
+
+  fd_dst = mkstemps(dst, strlen(ext));
+  if (fd_dst < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error creating tmp file '%s' for metadata update: %s\n", dst, strerror(errno));
+      goto error;
+    }
+
+  ret = fast_copy(fd_dst, fd_src);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error copying '%s' to tmp file '%s': %s\n", src, dst, strerror(errno));
+      goto error;
+    }
+
+  close(fd_src);
+  close(fd_dst);
+  return 0;
+
+ error:
+  if (fd_src != -1)
+    close(fd_src);
+  if (fd_dst != -1)
+    close(fd_dst);
+  return -1;
+}
+
+// based on FFmpeg's doc/examples and in particular mux.c
+static int
+file_write_rating(const char *dst, const char *src, const char *rating)
+{
+  AVFormatContext *in_fmt_ctx = NULL;
+  AVFormatContext *out_fmt_ctx = NULL;
+  AVPacket pkt;
+  const AVDictionaryEntry *tag;
+  AVStream *out_stream;
+  AVStream *in_stream;
+#if (LIBAVCODEC_VERSION_MAJOR > 59) || ((LIBAVCODEC_VERSION_MAJOR == 59) && (LIBAVCODEC_VERSION_MINOR >= 0) && (LIBAVCODEC_VERSION_MICRO >= 100))
+  const AVOutputFormat *out_fmt;
+#else
+  AVOutputFormat *out_fmt;
+#endif
+  bool restore_src = false;
+  int ret;
+  int i;
+  int stream_idx;
+  int *stream_mapping = NULL;
+
+  ret = avformat_open_input(&in_fmt_ctx, src, NULL, NULL);
+  if (ret != 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error opening tmpfile '%s' for rating metadata update: %s\n", src, av_err2str(ret));
+      goto error;
+    }
+
+  av_dict_set(&in_fmt_ctx->metadata, "rating", rating, 0);
+
+  ret = avformat_find_stream_info(in_fmt_ctx, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error reading input stream information from '%s': %s\n", in_fmt_ctx->url, av_err2str(ret));
+      goto error;
+    }
+
+  out_fmt = av_guess_format(in_fmt_ctx->iformat->name, in_fmt_ctx->url, in_fmt_ctx->iformat->mime_type);
+  if (out_fmt == NULL)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not determine output format from '%s'\n", in_fmt_ctx->url);
+      goto error;
+    }
+
+  ret = avformat_alloc_output_context2(&out_fmt_ctx, out_fmt, NULL, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not create output context '%s' - %s\n", in_fmt_ctx->url, av_err2str(ret));
+      goto error;
+    }
+
+  CHECK_NULL(L_SCAN, stream_mapping = av_calloc(in_fmt_ctx->nb_streams, sizeof(*stream_mapping)));
+
+  tag = NULL;
+  while ((tag = av_dict_iterate(in_fmt_ctx->metadata, tag)))
+    {
+      av_dict_set(&(out_fmt_ctx->metadata), tag->key, tag->value, 0);
+    }
+
+  stream_idx = 0;
+  for (i = 0; i < in_fmt_ctx->nb_streams; i++)
+    {
+      in_stream = in_fmt_ctx->streams[i];
+      stream_mapping[i] = stream_idx++;
+
+      out_stream = avformat_new_stream(out_fmt_ctx, NULL);
+      if (!out_stream)
+        {
+	  DPRINTF(E_LOG, L_SCAN, "Error allocating output stream for '%s'\n", in_fmt_ctx->url);
+	  goto error;
+        }
+
+      ret = avcodec_parameters_copy(out_stream->codecpar, in_stream->codecpar);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_SCAN, "Error copying codec parameters from '%s': %s\n", in_fmt_ctx->url, av_err2str(ret));
+	  goto error;
+	}
+
+      if (in_stream->metadata)
+	{
+	  tag = NULL;
+	  while ((tag = av_dict_iterate(in_stream->metadata, tag)))
+	    {
+	      av_dict_set(&(out_stream->metadata), tag->key, tag->value, 0);
+	    }
+	}
+    }
+
+  ret = avio_open(&out_fmt_ctx->pb, dst, AVIO_FLAG_WRITE);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Could not open output rating file '%s': %s\n", dst, av_err2str(ret));
+      goto error;
+    }
+
+  ret = avformat_write_header(out_fmt_ctx, NULL);
+  if (ret < 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Error occurred when writing output header to '%s': %s\n", dst, av_err2str(ret));
+      goto error;
+    }
+
+  while (1)
+    {
+      ret = av_read_frame(in_fmt_ctx, &pkt);
+      if (ret < 0)
+	{
+	  if (ret == AVERROR_EOF)
+	    break;
+
+	  DPRINTF(E_LOG, L_SCAN, "Error reading '%s': %s\n", in_fmt_ctx->url, av_err2str(ret));
+	  restore_src = true;
+	  goto error;
+	}
+
+      in_stream = in_fmt_ctx->streams[pkt.stream_index];
+      if (pkt.stream_index >= in_fmt_ctx->nb_streams || stream_mapping[pkt.stream_index] < 0)
+	{
+	  av_packet_unref(&pkt);
+	  continue;
+	}
+
+      pkt.stream_index = stream_mapping[pkt.stream_index];
+      out_stream = out_fmt_ctx->streams[pkt.stream_index];
+
+      /* copy packet */
+      pkt.pts = av_rescale_q_rnd(pkt.pts, in_stream->time_base, out_stream->time_base, AV_ROUND_NEAR_INF | AV_ROUND_PASS_MINMAX);
+      pkt.dts = av_rescale_q_rnd(pkt.dts, in_stream->time_base, out_stream->time_base, AV_ROUND_NEAR_INF | AV_ROUND_PASS_MINMAX);
+      pkt.duration = av_rescale_q(pkt.duration, in_stream->time_base, out_stream->time_base);
+      pkt.pos = -1;
+
+      ret = av_interleaved_write_frame(out_fmt_ctx, &pkt);
+      av_packet_unref(&pkt);
+      if (ret < 0)
+	{
+	  DPRINTF(E_LOG, L_SCAN, "Error muxing pkt for rating '%s': %s\n", in_fmt_ctx->url, av_err2str(ret));
+	  restore_src = true;
+	  goto error;
+	}
+    }
+
+  av_write_trailer(out_fmt_ctx);
+
+  if (out_fmt_ctx && !(out_fmt_ctx->oformat->flags & AVFMT_NOFILE))
+    avio_closep(&out_fmt_ctx->pb);
+  avformat_free_context(out_fmt_ctx);
+  av_freep(&stream_mapping);
+  return 0;
+
+ error:
+  if (out_fmt_ctx && !(out_fmt_ctx->oformat->flags & AVFMT_NOFILE))
+    avio_closep(&out_fmt_ctx->pb);
+  avformat_free_context(out_fmt_ctx);
+  av_freep(&stream_mapping);
+  if (restore_src)
+    file_copy(dst, src);
+  return -1;
+}
+
+static bool
+file_rating_matches(const char *path, const char *rating)
+{
+  AVFormatContext *in_fmt_ctx = NULL;
+  AVDictionaryEntry *entry;
+  bool has_rating;
+  int ret;
+
+  ret = avformat_open_input(&in_fmt_ctx, path, NULL, NULL);
+  if (ret != 0)
+    {
+      DPRINTF(E_LOG, L_SCAN, "Failed to open library file for rating metadata update '%s' - %s\n", path, av_err2str(ret));
+      return true; // Return true so called aborts
+    }
+
+  entry = av_dict_get(in_fmt_ctx->metadata, "rating", NULL, 0);
+  has_rating = (entry && entry->value && strcmp(entry->value, rating) == 0);
+
+  avformat_close_input(&in_fmt_ctx);
+
+  return has_rating;
+}
+
+// ffmpeg's metadata update is limited - some formats do not support rating
+// update even though the write completes; keep this in sync with supported
+// formats
+static bool
+format_is_supported(const char *format)
+{
+  if (strcmp(format, "mp3") == 0)
+    return true;
+  if (strcmp(format, "flac") == 0)
+    return true;
+
+  return false;
+}
+
+int
+write_metadata_ffmpeg(struct media_file_info *mfi)
+{
+  char rating_str[32];
+  char tmpfile[PATH_MAX];
+  int max_rating;
+  int file_rating;
+  int ret;
+
+  if (mfi->data_kind != DATA_KIND_FILE || !format_is_supported(mfi->type))
+    {
+      DPRINTF(E_WARN, L_SCAN, "Update of rating metadata requires file in MP3 or FLAC format: '%s'\n", mfi->path);
+      return -1;
+    }
+
+  max_rating = cfg_getint(cfg_getsec(cfg, "library"), "max_rating");
+  if (max_rating < 5) // Invalid config
+    max_rating = DB_FILES_RATING_MAX;
+  file_rating = mfi->rating * max_rating / DB_FILES_RATING_MAX;
+  snprintf(rating_str, sizeof(rating_str), "%d", file_rating);
+
+  // Save a write if metadata of the underlying file matches requested rating
+  if (file_rating_matches(mfi->path, rating_str))
+    return 0;
+
+  ret = file_copy_to_tmp(tmpfile, sizeof(tmpfile), mfi->path);
+  if (ret < 0)
+    return -1;
+
+  ret = file_write_rating(mfi->path, tmpfile, rating_str);
+  unlink(tmpfile);
+  if (ret < 0)
+    return -1;
+
+  DPRINTF(E_DBG, L_SCAN, "Wrote rating metadata to '%s'\n", mfi->path);
+
+  return 0;
+}
