@@ -34,7 +34,8 @@
 #include "misc.h"
 
 
-static struct lws_context *context;
+static struct lws_context *websocket_context;
+static bool websocket_is_initialized;
 static pthread_t tid_websocket;
 
 static const char *websocket_interface;
@@ -47,15 +48,15 @@ static pthread_mutex_t websocket_write_event_lock;
 static short websocket_write_events;
 
 
-/* Thread: library (the thread the event occurred) */
+/* Thread: library, player, etc. (the thread the event occurred) */
 static void
-listener_cb(short event_mask)
+listener_cb(short event_mask, void *ctx)
 {
   pthread_mutex_lock(&websocket_write_event_lock);
   websocket_write_events |= event_mask;
   pthread_mutex_unlock(&websocket_write_event_lock);
 
-  lws_cancel_service(context);
+  lws_cancel_service(websocket_context);
 }
 
 /*
@@ -132,7 +133,7 @@ process_notify_request(short *requested_events, void *in, size_t len)
 	  if (json_object_get_type(item) == json_type_string)
 	    {
 	      event_type = json_object_get_string(item);
-	      DPRINTF(E_DBG, L_WEB, "notify callback event received: %s\n", event_type);
+	      DPRINTF(E_SPAM, L_WEB, "notify callback event received: %s\n", event_type);
 
 	      if (0 == strcmp(event_type, "update"))
 		{
@@ -278,7 +279,7 @@ callback_notify(struct lws *wsi, enum lws_callback_reasons reason, void *user, v
   short events = 0;
   int ret = 0;
 
-  DPRINTF(E_DBG, L_WEB, "notify callback reason: %d\n", reason);
+  DPRINTF(E_SPAM, L_WEB, "notify callback reason: %d\n", reason);
 
   switch (reason)
   {
@@ -415,20 +416,22 @@ static void *
 websocket(void *arg)
 {
   listener_add(listener_cb, LISTENER_UPDATE | LISTENER_DATABASE | LISTENER_PAIRING | LISTENER_SPOTIFY | LISTENER_LASTFM | LISTENER_SPEAKER
-               | LISTENER_PLAYER | LISTENER_OPTIONS | LISTENER_VOLUME | LISTENER_QUEUE);
+               | LISTENER_PLAYER | LISTENER_OPTIONS | LISTENER_VOLUME | LISTENER_QUEUE, NULL);
 
   while(!websocket_exit)
   {
 #if LWS_LIBRARY_VERSION_MAJOR >= 3
-    if (lws_service(context, 0))
+    if (lws_service(websocket_context, 0))
       websocket_exit = true;
 #else
-    lws_service(context, 10000);
+    lws_service(websocket_context, 10000);
     if (websocket_write_events)
-      lws_callback_on_writable_all_protocol(context, &protocols[WS_PROTOCOL_NOTIFY]);
+      lws_callback_on_writable_all_protocol(websocket_context, &protocols[WS_PROTOCOL_NOTIFY]);
 #endif
   }
 
+  listener_remove(listener_cb);
+
   pthread_exit(NULL);
 }
 
@@ -475,7 +478,11 @@ websocket_init(void)
 
   if (websocket_port <= 0)
     {
+#ifdef HAVE_LIBEVENT22
+      DPRINTF(E_DBG, L_WEB, "Libwebsocket disabled, using libevent websocket instead. To enable it, set websocket_port in config to a valid port number.\n");
+#else
       DPRINTF(E_LOG, L_WEB, "Websocket disabled. To enable it, set websocket_port in config to a valid port number.\n");
+#endif
       return 0;
     }
 
@@ -502,8 +509,8 @@ websocket_init(void)
   lws_set_log_level(LLL_ERR | LLL_WARN | LLL_NOTICE | LLL_INFO | LLL_DEBUG,
                     logger_libwebsockets);
 
-  context = lws_create_context(&info);
-  if (context == NULL)
+  websocket_context = lws_create_context(&info);
+  if (websocket_context == NULL)
     {
       DPRINTF(E_LOG, L_WEB, "Failed to create websocket context\n");
       return -1;
@@ -513,7 +520,7 @@ websocket_init(void)
   if (ret < 0)
     {
       DPRINTF(E_LOG, L_WEB, "Failed to initialize mutex: %s\n", strerror(ret));
-      lws_context_destroy(context);
+      lws_context_destroy(websocket_context);
       return -1;
     }
 
@@ -522,12 +529,14 @@ websocket_init(void)
     {
       DPRINTF(E_LOG, L_WEB, "Could not spawn websocket thread (%d): %s\n", ret, strerror(ret));
       pthread_mutex_destroy(&websocket_write_event_lock);
-      lws_context_destroy(context);
+      lws_context_destroy(websocket_context);
       return -1;
     }
 
   thread_setname(tid_websocket, "websocket");
 
+  websocket_is_initialized = true;
+
   return 0;
 }
 
@@ -536,15 +545,16 @@ websocket_deinit(void)
 {
   int ret;
 
-  if (websocket_port <= 0)
+  if (!websocket_is_initialized)
     return;
 
+  websocket_is_initialized = false;
   websocket_exit = true;
-  lws_cancel_service(context);
+  lws_cancel_service(websocket_context);
   ret = pthread_join(tid_websocket, NULL);
   if (ret < 0)
     DPRINTF(E_LOG, L_WEB, "Error joining websocket thread (%d): %s\n", ret, strerror(ret));
 
-  lws_context_destroy(context);
+  lws_context_destroy(websocket_context);
   pthread_mutex_destroy(&websocket_write_event_lock);
 }
